/**
 * Token
 */
import ext from './extend.js';

// 默认过期时间 8 小时
const defaultExpriesIn = 28800000;

export default {
    /**
     * 判断 Token 是否生效
     * @returns {boolean} 存在标记
     */
    has() {
        let token = ext.readStorage('token');
        let timeout = ext.readStorage('token_timeout');
        let now = Date.now();

        if (timeout < now || ext.isEmptyString(token)) {
            this.clear();
            return false;
        }
        return true;
    },

    /**
     * 设置 Token
     * @param {Object} param - 参数
     * @param  {string} param.token - 令牌
     * @param {number} param.expries_in - 生效时长，默认 8 小时
     */
    set(param) {
        if (!ext.isObject(param) || ext.isEmptyString(param.token)) {
            return;
        }
        let timeout = 0;
        if (ext.isNumber(param.expries_in)) {
            timeout = Date.now() + param.expries_in;
        } else {
            timeout = Date.now() + defaultExpriesIn;
        }
        ext.addStorage('token', param.token);
        ext.addStorage('token_timeout', timeout);
    },

    /**
     * 获取 Token
     */
    get() {
        this.has();
        return ext.readStorage('token');
    },

    /**
     * 清除 Token
     */
    clear() {
        ext.removeStorage('token');
        ext.removeStorage('token_timeout');
    }
};