const app = getApp();
const util = require("../../../utils/util");
var allList = [];
var selectCount = 0;
var searchText = '';

Page({

	/**
	 * 页面的初始数据
	 */
	data: {
		StatusBar: app.globalData.StatusBar,
		CustomBar: app.globalData.CustomBar,
		Custom: app.globalData.Custom,
		user: {},
		searchText: '',
		pageIndex: 0,
		pageSize: 500,
		rowsCount: 0,
		list: [],
		isLoad: false,
		loadType: "loading",
		loadText: "加载中...",
		selectedIndex: 0,
		queryType: 10,

		// 表格是否全选
		allChecked: false,
	},

	/**
	 * 生命周期函数--监听页面加载
	 */
	onLoad: function () {
		searchText = '';
		this.Load();
	},

	/**
	 * 生命周期函数--监听页面显示
	 */
	onShow: function () {
		var page = this;
		var now = new Date();
		selectCount = 0;
		this.setData({
			loadType: 'loading',
			list: [],
			queryType: 10,
			isJoin: true,
			allChecked: false
		});
	},
	/**
	 * 搜索框输入事件
	 */
	OnCheckbox: function (e) {
		var i = 0;
		var key = '';
		var checked = e.currentTarget.dataset.checked == "true" || e.currentTarget.dataset.checked ? false : true;
		var list = this.data.list;
		switch (e.currentTarget.dataset.type) {
			case 'all':
				for (i = 0; i < list.length; i++) {
					list[i].checked = checked;
				}
				selectCount = checked ? list.length : 0;
				this.setData({
					allChecked: checked,
					list: list
				});
				break;
			case 'table':
			default:
				list[e.currentTarget.dataset.index].checked = checked;
				checked ? selectCount++ : selectCount--;
				this.setData({
					allChecked: selectCount === list.length,
					list: list
				});
		}
	},

	/**
	 * 选择
	 */
	Select: function () {
		var list = [];
		for (var i = 0; i < this.data.list.length; i++) {
			if (this.data.list[i].checked) {
				list.push(this.data.list[i]);
			}
		}
		if (list.length === 0) {
			return wx.showToast({
				title: '请勾选组长',
				icon: 'none',
			});
		}
		wx.setStorageSync('select', {
			name: 'departmentWorkGroup',
			data: list
		});
		wx.navigateBack();
	},

	/**
	 * 改变查询类型
	 */
	ChangeQueryType: function (e) {
		var queryType = parseInt(e.currentTarget.dataset.type);
		if (!isNaN(queryType)) {
			this.setData({
				queryType: queryType
			});
			this.Reload();
		}
	},

	/**
	 * 搜索框输入事件
	 */
	OnSearchInput: function (e) {
		searchText = e.detail.value
	},

	/**
	 * 清除查询
	 */
	ClearSearchInput: function () {
		this.setData({
			searchText: ''
		});
		searchText = '';
		this.Search();
	},

	/**
	 * 跳页
	 * @param {object} e 事件对象 
	 */
	GotoPage: function (e) {
		var pageIndex = parseInt(e.currentTarget.dataset.pageindex);
		if (isNaN(pageIndex)) {
			return;
		}
		this.Search(pageIndex);
	},

	/**
	 * 查询
	 * @param {number} pageIndex 
	 */
	Search: function (pageIndex) {
		pageIndex = util.numberIsNullOrEmpty(pageIndex) ? 1 : pageIndex < 1 ? 1 : pageIndex;
		// 分页后的数据
		var list = [];
		// 筛选后的数据
		var slist = [];
		if (!util.stringIsNullOrWhiteSpace(searchText)) {
			for (var i = 0; i < allList.length; i++) {
				for (var name in allList[i]) {
					if (!util.objectIsNullOrEmpty(allList[i][name]) && allList[i][name].toString().indexOf(searchText) >= 0) {
						slist.push(allList[i]);
						break;
					}
				}
			}
		}
		else {
			slist = allList;
		}
		var start = (pageIndex - 1) * this.data.pageSize;
		var end = pageIndex * this.data.pageSize > slist.length ? slist.length : pageIndex * this.data.pageSize;
		for (start; start < end; start++) {
			list.push(slist[start]);
		}
		this.setData({
			list: list,
			pageIndex: pageIndex,
			rowsCount: slist.length,
			isLoad: false
		});
	},

	/**
	 * 加载数据
	 */
	Load: function (e) {
		if (this.data.isLoad || this.data.loadType === 'over') {
			return false;
		}
		this.setData({
			isLoad: true,
			loadText: "加载中...",
		});
		var page = this;
		var pageIndex = this.data.pageIndex + 1;
		app.Request({
			url: app.GetDomain() + '/api/DepartmentWorkGroup/GetDepartmentEmpAndWorkGroupLeaderInfoForWeb',
			method: 'get',
			data: {
				queryType: page.data.queryType
			},
			success: res => {
				allList = util.arrayIsNullOrEmpty(res) ? [] : res;
				page.Search();
			},
			fail: res => {
				page.setData({
					isLoad: false,
					loadType: 'erro'
				});
				wx.showToast({
					title: res.errMsg,
					icon: 'none',
				});
			}
		});
	},

	/**
	 * 重新加载
	 */
	Reload: function (e) {
		this.setData({
			list: [],
			pageIndex: 0
		});
		this.Load();
	},
})