const app = getApp();
const util = require("../../../utils/util");

Page({
    /**
     * 页面的初始数据
     */
    data: {
        StatusBar: app.globalData.StatusBar,
        CustomBar: app.globalData.CustomBar,
        Custom: app.globalData.Custom,
		user: {},
		column: 2,
		notices: [],
		noticeOption: {
			pageIndex: 1,
			pageSize: 5,
			isLoad: false,
			isLast: false,
			loadText: ' 加载中...'
		},
		menus: []
    },

    /**
     * 生命周期函数--监听页面加载
     */
	onLoad: function (options) {
        if (!app.globalData.isLogin) {
            this.data.loadText = '未登录';
            return;
        }
		var page = this;
		var noticeOption = page.data.noticeOption;
		noticeOption.pageIndex = 1;
		noticeOption.isLoad = false;
		this.setData({
			noticeOption: noticeOption
		});
		app.GetUser({
			success: res => {
				page.setData({
					user: res
				});
				page.showMenu();
				page.LoadNotices();
			},
			fail: res => {
				wx.showToast({
					title: res.errMsg,
					icon: 'none'
				});
			}
		});
		wx.getSystemInfo({
			success: res => {
				var column = 2;
				if (res.screenWidth >= 768) {
					column = 4;
				}
				else {
					column = 2;
				}
				page.setData({
					column: column
				});
			}
		});
	},

    /**
     * 生命周期函数--监听页面显示
     */
    onShow: function() {
		wx.setStorageSync('select', null);
		wx.setStorageSync('workGroupMember', null);
		var noticeOption = this.data.noticeOption;
		noticeOption.pageIndex = 1;
		this.setData({
			notices: [],
			noticeOption: noticeOption
		});
    },

	/**
	 * 调整通知页面
	 */
	GotoNotice: function (e) {
		for (var i = 0; i < this.data.notices.length; i++) {
			if (this.data.notices[i].id === e.currentTarget.dataset.id) {
				return wx.navigateTo({
					url: '/pages/notices/list/list',
					success: res => {
						res.eventChannel.emit('sendNotice', this.data.notices[i]);
					}
				});
			}
		}
	},

	/**
	 * 加载通知
	 */
	LoadNotices: function (e) {
		if (this.data.noticeOption.isLoad) {
			return false;
		}
		var page = this;
		var noticeOption = this.data.noticeOption;
		noticeOption.isLoad = true;
		noticeOption.loadText = ' 加载中...'
		this.setData({
			noticeOption: noticeOption
		});
		app.Request({
			url: app.GetDomain() + '/api/Notice/GetCreateDatePagedByCurrentUser',
			method: 'get',
			data: {
				Skip: (page.data.noticeOption.pageIndex - 1) * page.data.noticeOption.pageSize,
				Take: page.data.noticeOption.pageSize
			},
			success: res => {
				if (util.arrayIsNullOrEmpty(res.items)) {
					res.items = [];
				}
				var noticeOption = page.data.noticeOption;
				noticeOption.isLoad = false;
				noticeOption.pageIndex++;
				if (res.items.length < page.data.noticeOption.pageSize) {
					noticeOption.isLast = true;
					noticeOption.loadText = '已经到底了';
				}
				else {
					noticeOption.loadText = '点击加载更多';
				}

				// 格式化数据
				for (var i = 0; i < res.items.length; i++) {
					res.items[i].createOnDateText = util.formatDate(util.parseDate(res.items[i].createOnDate), 'yyyy年MM月dd日');
					res.items[i].createOnDateParam = util.formatDate(util.parseDate(res.items[i].createOnDate), 'yyyy-MM-dd');
				}
				page.data.notices = page.data.notices.concat(res.items);
				if (page.data.notices.length === 0) {
					noticeOption.loadText = '没有记录';
				}
				page.setData({
					notices: page.data.notices,
					noticeOption: noticeOption
				});
				var _timer = setTimeout(() => {
					page.LoadNotices();
					clearTimeout(_timer);
				}, 60000 * 5);
			},
			fail: res => {
				page.setData({
					isLoad: false,
					loadType: 'erro'
				});
				wx.showModal({
					title: '应用程序错误',
					content: res.errMsg,
				});
			}
		});
	},

	/**
	 * 显示菜单
	 */
	showMenu: function () {
		var menus = [];
		/**
		 * 角色为空，显示的菜单：
		 * 我的派工。
		 * 角色不为空，显示所有菜单。
		 */
		if (util.arrayIsNullOrEmpty(this.data.user.roles) || this.data.user.roles.length === 0) {
			menus = this.getMenus('none');
		}
		else {
			menus = this.getMenus();
		}
		this.setData({
			menus: menus
		});
	},

	/**
	 * 获取菜单
	 * @param {string} findName 菜单名查询字符串
	 */
	getMenus: function (findName) {
		var result = [];
		if (findName === 'none') {
			return result;
		}
		var menus = [
			{
				name: '主管派工',
				bgColorClass: 'bg-cyanLight',
				url: '/pages/workshopSupervisorDispatch/index/index'
			},
			{
				name: '组长派工',
				bgColorClass: 'bg-cyanLight',
				url: '/pages/workGroupLeaderDispatch/index/index'
			},
			{
				name: '我的派工',
				bgColorClass: 'bg-olive',
				url: '/pages/manualDisptch/index/index'
			},
			{
				name: '报工申请',
				bgColorClass: 'bg-olive',
				url: '/pages/workOrderApplication/index/index'
			},
			{
				name: '流程生产查询',
				bgColorClass: 'bg-olive',
				url: '/pages/report/mesProductionOrderProcess/index'
			},
			{
				name: '生产进度明细',
				bgColorClass: 'bg-olive',
				url: '/pages/report/productionProgressDetails/index'
			},
			{
				name: '生产进度',
				bgColorClass: 'bg-olive',
				url: '/pages/report/productionProgress/index'
			}
		];
		if (util.stringIsNullOrWhiteSpace(findName)) {
			return menus;
		}
		for (var i = 0; i < menus.length; i++) {
			if (menus[i].name.indexOf(findName) >= 0) {
				result.push(menus[i]);
			}
		}
		return result;
	}
});