/**
 * 触发事件
 */
function triggerEvent(option) {
	if (typeof option !== "object") {
		option = {};
	}
	var myEventDetail = {
		component: option.component || "page",
		type: option.type || "navChange",
		data: option.data || {
			pageCur: option.pageCur || "home"
		}
	} // detail对象，提供给事件监听函数
	if (typeof option.page !== 'object') {
		option.page = getCurrentPages()[0];
	}
	option.page.triggerEvent(option.event || "myevent", myEventDetail, option.myEventOption || {});
}

const formatTime = date => {
	const year = date.getFullYear();
	const month = date.getMonth() + 1;
	const day = date.getDate();
	const hour = date.getHours();
	const minute = date.getMinutes();
	const second = date.getSeconds();

	return [year, month, day].map(formatNumber).join('/') + ' ' + [hour, minute, second].map(formatNumber).join(':');
}

const formatNumber = n => {
	n = n.toString();
	return n[1] ? n : '0' + n;
}

/**
 * 判断空对象
 */
const objectIsNullOrEmpty = function (value) {
	return value === null || value === undefined;
}

/**
 * 格式化金额
 */
const formatAmount = function (value, length) {
	if (!(typeof value === 'number' && !isNaN(value))) return '-';
	typeof length === 'number' && !isNaN(length) ? true : length = -1;
	value = floatRound(value, length);
	if (length === 0) value = Math.round(value);
	var text = value.toString(),
		temp = '',
		end = '',
		sign = '';
	if (text.substring(0, 1) === '-') {
		sign = '-';
		text = text.substring(1);
	}
	// 格式化整数部分
	if (text.indexOf('.') > 0) text = text.substring(0, text.indexOf('.'));
	var _length = text.length;
	for (var i = 0; i < _length; ++i) {
		temp = text.substring(text.length - 1);
		text = text.substring(0, text.length - 1);
		if (temp.length > 0)
			if ((i + 1) % 3 === 0 && text.length > 0) end = ',' + temp + end;
			else end = temp + end;
		else break;
	}
	// 格式化小数部分
	text = value.toString();
	if (text.indexOf('.') > 0) {
		end += '.';
		text = text.substring(text.indexOf('.') + 1);
	} else {
		text = '';
		if (length > 0) end += '.';
	}
	if (length < 0) length = text.length;
	for (i = 0; i < length; ++i) {
		if (text.length < 1) temp = '0';
		else {
			temp = text.substring(0, 1);
			text = text.substring(1);
		}
		end += temp;
	}
	return sign + end;
}

/**
 * 浮点数四舍五入
 */
const floatRound = function (value, length) {
	if (!(typeof value === 'number' && !isNaN(value))) return value;
	typeof length === 'number' && !isNaN(length) ? true : length = 2;
	if (length > 0) {
		value = Math.round(value * Math.pow(10, length)) / Math.pow(10, length);
	}
	return value;
}

/**
 * 字符串前缀拼接
 * value 值的长度 length 不够时，将以前缀 prefix 填充，比如：001
 */
const stringPrefix = function (value, prefix, length) {
	if (!numberIsNullOrEmpty(value)) {
		value = value.toString();
	}
	if (stringIsNullOrEmpty(value) || stringIsNullOrEmpty(prefix) || numberIsNullOrEmpty(length)) {
		return '';
	} else {
		return (Array(length).join(prefix) + value).slice(-length);
	}
}

/**
 * 判断是否为空字符串
 */
const stringIsNullOrEmpty = function (value) {
	return objectIsNullOrEmpty(value) || typeof value != 'string';
}

/**
 * 判断是否为空或空白字符串
 */
const stringIsNullOrWhiteSpace = function (value) {
	if (typeof value === 'string') {
		value = value.replace(/ /g, '');
		if (value.length > 0) {
			return false;
		}
	}
	return true;
}

/**
 * 判断空数字
 */
const numberIsNullOrEmpty = function (value) {
	return typeof value !== 'number' || isNaN(value);
}

/**
 * 判断日期类型
 */
const isDate = function (value) {
	return !(value instanceof Date) || value.toString() === "Invalid Date";
}

/**
 * 判断是否是空数组
 */
const arrayIsNullOrEmpty = function (value) {
	return !(value instanceof Array || (!objectIsNullOrEmpty(value) && !numberIsNullOrEmpty(value.length) && stringIsNullOrEmpty(value)));
}

/**
 * 日期格式化
 */
const formatDate = function (value, format) {
	if (objectIsNullOrEmpty(value)) {
		return '';
	}
	if (isDate(value)) {
		return formatDate(parseDate(value), format) || '';
	} else {
		var hours = value.getHours();
		format = stringIsNullOrWhiteSpace(format) ? 'yyyy-MM-dd HH:mm:ss.ms' : format;
		format = format.replace(/yyyy|YYYY/, value.getFullYear().toString());
		format = format.replace(/yy|YY/, stringPrefix(value.getYear() % 100, '0', 2));
		format = format.replace(/MM/, stringPrefix(value.getMonth() + 1, '0', 2));
		format = format.replace(/M/g, (value.getMonth() + 1).toString());
		format = format.replace(/dd|DD/, stringPrefix(value.getDate(), '0', 2));
		format = format.replace(/d|D/g, value.getDate().toString());
		format = format.replace(/HH/g, stringPrefix(hours, '0', 2));
		format = format.replace(/hh/g, hours >= 12 ? '下午' + stringPrefix(hours - 12, '0', 2) : '上午' + stringPrefix(hours, '0', 2));
		format = format.replace(/H/g, hours);
		format = format.replace(/h/g, hours >= 12 ? '下午' + (hours - 12).toString() : '上午' + hours.toString());
		format = format.replace(/mm/g, stringPrefix(value.getMinutes(), '0', 2));
		format = format.replace(/m/g, value.getMinutes().toString());
		format = format.replace(/ss/g, stringPrefix(value.getSeconds(), '0', 2));
		format = format.replace(/s/g, value.getSeconds().toString());
		format = format.replace(/ms/g, value.getMilliseconds().toString());
		return format;
	}
}

/**
 * 日期字符串转换成日期对象
 */
const parseDate = function (value) {
	var newDate = null;
	if (objectIsNullOrEmpty(value)) {
		return newDate;
	}
	if (!numberIsNullOrEmpty(value)) {
		value = 'Date(' + value + ')';
	}
	if (!stringIsNullOrWhiteSpace(value)) {
		if (value.indexOf('Date') > 0) {
			value = "new " + value.replace(/\//g, '');
		}
		value = value.replace(/-/g, '/');
		value = value.replace(/T/g, ' ');
		if (value.indexOf('Date') >= 0) {
			newDate = eval(value);
		} else {
			var ms = 0;
			var msIndex = value.indexOf('.');
			if (msIndex > -1) {
				var ms = parseInt(value.substring(msIndex + 1));
				value = value.substring(0, msIndex);
			}
			newDate = new Date(value);
			if (!isNaN(ms)) {
				newDate.setMilliseconds(ms);
			}
		}
	}
	return newDate;
}

/**
 * 格式化错误
 */
const formatError = function (ex) {
	var result = {
		status: null,
		success: false,
		errMsg: ''
	};
	if (typeof ex === 'string') {
		result.errMsg = ex;
	} else if (typeof ex === 'object') {
		if (!stringIsNullOrEmpty(ex.stack)) {
			result.errMsg = ex.stack;
		} else if (!stringIsNullOrEmpty(ex.responseText)) {
			result.errMsg = ex.responseText;
		} else if (!stringIsNullOrEmpty(ex.statusText)) {
			result.errMsg = ex.statusText;
		} else if (!stringIsNullOrEmpty(ex.title)) {
			result.errMsg = ex.title;
		} else if (!stringIsNullOrEmpty(ex.data)) {
			result.errMsg = ex.data;
		} else if (!objectIsNullOrEmpty(ex.data)) {
			if (!stringIsNullOrEmpty(ex.data.error)) {
				result.errMsg = ex.data.error;
			}
		} else if (!stringIsNullOrEmpty(ex.errMsg)) {
			result.errMsg = ex.errMsg;
		}
	}
	var statusMessage = {
		0: '网络连接失败，请检查本地网络设置。',
		400: '请求无效。',
		401: '未授权。',
		403: '禁止访问。',
		404: '访问地址不存在。',
		405: '不允许此方法。',
		406: '不可接受。',
		407: '需要代理身份验证。',
		412: '前提条件失败。',
		414: 'Request-URI 太长',
		415: '不支持的媒体类型。',
		500: '内部服务器错误。',
		502: '网关错误。',
		600: '不可解析的响应标头。',
	};
	if (!numberIsNullOrEmpty(ex.statusCode)) {
		result.status = ex.statusCode;
		if (result.status !== 200 && (stringIsNullOrWhiteSpace(result.errMsg) || result.errMsg === 'error')) {
			result.errMsg = result.status.toString() + ' ' + (statusMessage[result.status] || '');
		}
	}
	return result;
}

/**
 * 空对象格式化
 */
const emptyFormatter = function (value) {
	if (value instanceof Array) {
		for (var i = 0; i < value.length; ++i) {
			for (var j in value[i]) {
				value[i][j] == null ? value[i][j] = "" : true;
				typeof value[i][j] === "undefined" ? value[i][j] = "" : true;
			}
		}
	} else if (typeof value === 'object') {
		for (var i in value) {
			value[i] === null ? value[i] = "" : true;
			typeof value[i] === "undefined" ? value[i] = "" : true;
		}
	}
	return value;
}



/**
 * 将对象编码成 web 服务器能接收的数据类型
 */
function encodeObject(value) {
	var result = {};
	var name = '';

	Parse(value, name);

	return result;

	// 转换方法
	function Parse(value, name) {
		if (value instanceof Array) {
			for (var i = 0; i < value.length; ++i) {
				Parse(value[i], name + '[' + i + ']');
			}
		} else if (typeof value === 'object') {
			for (var i in value) {
				if (value[i] instanceof Array || typeof value[i] === 'object') {
					Parse(value[i], name.length > 0 ? (name + '.' + i) : i);
				} else if (name.length > 0) {
					result[name + '.' + i] = value[i];
				} else {
					result[i] = value[i];
				}
			}
		} else {
			result[name] = value;
		}
	}
}

module.exports = {
	triggerEvent: triggerEvent,
	objectIsNullOrEmpty: objectIsNullOrEmpty,
	formatTime: formatTime,
	formatAmount: formatAmount,
	floatRound: floatRound,
	stringPrefix: stringPrefix,
	stringIsNullOrEmpty: stringIsNullOrEmpty,
	stringIsNullOrWhiteSpace: stringIsNullOrWhiteSpace,
	numberIsNullOrEmpty: numberIsNullOrEmpty,
	isDate: isDate,
	arrayIsNullOrEmpty: arrayIsNullOrEmpty,
	formatDate: formatDate,
	parseDate: parseDate,
	formatError: formatError,
	emptyFormatter: emptyFormatter,
	encodeObject: encodeObject
}